#include "templ.h"
#include "Client.h"
#include "globus_toolkit.h"
#include "../GramServer/Server.h"

// Register all server side methods and functions.
// Call gs_init() to initialize Globus based HPC++. Very important!!!
void Client::initialize() {
  // register all the required functions
  Server::registerAll();

  // Initialize Globus. This must be done only once in the context.
  int returnValue = gs_init();

  if(returnValue) {
    cout << "Client:: gs_init() failed with return value " << returnValue << endl;
    cout << ".. Exiting ...  \n";
    exit(EXIT_FAILURE);
  } else {
    cout << "Client :: initialize() : gs_init() succeded \n";
  }
}


void Client::instantiateComponent(HPCxx_String gatekeeperMachine, 
							 HPCxx_String machineToRunOn, HPCxx_String path) {

  // Print out the command line arguments received
  printRequest(gatekeeperMachine, machineToRunOn, path);

  // Make sure you can connect to the Globus Gatekeeper. In the Globus
  // based HPC++, the Gatekeeper process running on the target machine is
  // used as the process creation daemon. This call makes sure that you
  // have the right authority to connect to the Gatekeeper.
  hpcxx_connectContext(gatekeeperMachine); 

  // Create a context on the target machine, and get its contextID
  // back. This contextID will be used to make the first invocation on the
  // newly created server context. The first invocation will return a
  // global pointer to a server object on the server's machine, so that the 
  // client can access the server object.
  HPCxx_ContextID *remoteCID = hpcxx_createContext(machineToRunOn, path);
  
  // If remote process' contextID is null, exit ...
  if (!remoteCID)  {
    cout << "NULL remoteCID  received by the Gramtest client\n";
    cout << "... Exiting ... \n";
    exit(EXIT_FAILURE);
  }

  cout << "Client :: received context ID of server process " << endl;

  HPCxx_GlobalPtr<Server> serverGp;
  // Invoke returnServerGp, 
  // Notice that when making a remote invocation using a contextId, you use
  // hpcxx_invoke, while for that usin a GP, you use hpcxx_minvoke, which is 
  //     ^^^                                             ^^^^
  // illustrated later.
  hpcxx_invoke(*remoteCID, serverGp, RETURN_SERVER_GP_ID);
  cout << "invoke for returnServerGp returned\n";

  cout << "trying out an invocation on the server GP\n";
    // Invoke the sayHello() method on the server
  HPCxx_String returnString;
  HPCxx_String argumentString("helloWorld from Client");

  cout << "Client :: Invoking sayHello() on server   " << endl;
  // Again, for remote request using a global pointer, use hpcxx_minvoke.
  hpcxx_minvoke(&serverGp, returnString, SAY_HELLO_ID, argumentString);
  
  cout << "Client :: Server replied : " << returnString << endl;
  cout << "Client :: Invoking killMe on server   " << endl;

  // Kill the server by calling killMe on it. It returns some dummy value
  // back to the client.
  int x;
  hpcxx_minvoke(&serverGp, x, KILL_ME_ID);
  cout << "return value from killMe = " << x << endl;

  // Get rid of the context ID for the server context
  // hpcxx_destroyContextID(remoteCID);
}

// Print out the ommand line arguments for the client.
void Client::printRequest(HPCxx_String gateKeeperMachine, HPCxx_String machineToRunOn, 
					   HPCxx_String path) {
  cout << "\n Component::printRequest " << endl;
  cout << "\n printRequest::gateKeeperMachine: " <<  gateKeeperMachine << endl;
  cout << "\n printRequest::machineToRunOn: " << machineToRunOn << endl;
  cout << "\n printRequest::executablePath: " << path << endl;
}
